@@ -4,7 +4,17 @@ import sys
 import functools
 
 from types import MethodType
-from typing import Any, Dict, Optional, Type, TypeVar, Callable, TYPE_CHECKING
+from typing import (
+    Any,
+    Dict,
+    Optional,
+    Type,
+    TypeVar,
+    Callable,
+    TYPE_CHECKING,
+    ParamSpec,
+    overload,
+)
 from datetime import timedelta
 from contextlib import asynccontextmanager
 
@@ -32,10 +42,12 @@ from mcp_agent.utils.common import unwrap
 from mcp_agent.workflows.llm.llm_selector import ModelSelector
 from mcp_agent.workflows.factory import load_agent_specs_from_dir
 
+
 if TYPE_CHECKING:
     from mcp_agent.agents.agent_spec import AgentSpec
     from mcp_agent.executor.workflow import Workflow
 
+P = ParamSpec("P")
 R = TypeVar("R")
 
 
@@ -89,10 +101,6 @@ class MCPApp:
             initialize_model_selector: Initializes the built-in ModelSelector to help with model selection. Defaults to False.
         """
         self.mcp = mcp
-        self.name = name or (mcp.name if mcp else None)
-        self.description = description or (
-            mcp.instructions if mcp else "MCP Agent Application"
-        )
 
         # We use these to initialize the context in initialize()
         if settings is None:
@@ -102,6 +110,14 @@ class MCPApp:
         else:
             self._config = settings
 
+        self.name = name or self._config.name or (mcp.name if mcp else None)
+
+        self.description = (
+            description
+            or self._config.description
+            or (mcp.instructions if mcp else "MCP Agent Application")
+        )
+
         # We initialize the task and decorator registries at construction time
         # (prior to initializing the context) to ensure that they are available
         # for any decorators that are applied to the workflow or task methods.
@@ -448,6 +464,7 @@ class MCPApp:
             decorated_cls = workflow_defn_decorator(
                 cls, sandboxed=False, *args, **kwargs
             )
+
             self._workflows[workflow_id] = decorated_cls
             return decorated_cls
         else:
@@ -649,6 +666,8 @@ class MCPApp:
             return res
 
         async def _run(self, *args, **kwargs):  # type: ignore[no-redef]
+            # ensure initialization
+            await self.initialize()
             return await _invoke_target(self, *args, **kwargs)
 
         # Decorate run with engine-specific decorator
@@ -712,13 +731,25 @@ class MCPApp:
         self.workflow(auto_cls, workflow_id=workflow_name)
         return auto_cls
 
+    @overload
+    def tool(self, __fn: Callable[P, R]) -> Callable[P, R]: ...
+
+    @overload
     def tool(
         self,
         name: str | None = None,
         *,
         description: str | None = None,
         structured_output: bool | None = None,
-    ) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
+    ) -> Callable[[Callable[P, R]], Callable[P, R]]: ...
+
+    def tool(
+        self,
+        name: str | None = None,
+        *,
+        description: str | None = None,
+        structured_output: bool | None = None,
+    ):
         """
         Decorator to declare a synchronous MCP tool that runs via an auto-generated
         Workflow and waits for completion before returning.
@@ -727,7 +758,7 @@ class MCPApp:
         endpoints are available.
         """
 
-        def decorator(fn: Callable[..., Any]) -> Callable[..., Any]:
+        def decorator(fn: Callable[P, R]) -> Callable[P, R]:
             tool_name = name or fn.__name__
 
             # Early validation: Use the shared tool adapter logic to validate
@@ -760,18 +791,29 @@ class MCPApp:
 
         # Support bare usage: @app.tool without parentheses
         if callable(name) and description is None and structured_output is None:
-            fn = name  # type: ignore[assignment]
+            _fn = name  # type: ignore[assignment]
             name = None
-            return decorator(fn)  # type: ignore[arg-type]
+            return decorator(_fn)  # type: ignore[arg-type]
 
         return decorator
 
+    @overload
+    def async_tool(self, __fn: Callable[P, R]) -> Callable[P, R]: ...
+
+    @overload
+    def async_tool(
+        self,
+        name: str | None = None,
+        *,
+        description: str | None = None,
+    ) -> Callable[[Callable[P, R]], Callable[P, R]]: ...
+
     def async_tool(
         self,
         name: str | None = None,
         *,
         description: str | None = None,
-    ) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
+    ):
         """
         Decorator to declare an asynchronous MCP tool.
 
@@ -779,7 +821,7 @@ class MCPApp:
         the standard per-workflow tools (run/get_status) are exposed by the server.
         """
 
-        def decorator(fn: Callable[..., Any]) -> Callable[..., Any]:
+        def decorator(fn: Callable[P, R]) -> Callable[P, R]:
             workflow_name = name or fn.__name__
 
             # Early validation: Use the shared tool adapter logic to validate
@@ -810,9 +852,9 @@ class MCPApp:
 
         # Support bare usage: @app.async_tool without parentheses
         if callable(name) and description is None:
-            fn = name  # type: ignore[assignment]
+            _fn = name  # type: ignore[assignment]
             name = None
-            return decorator(fn)  # type: ignore[arg-type]
+            return decorator(_fn)  # type: ignore[arg-type]
 
         return decorator
 
