@@ -4,11 +4,13 @@ A central context object to store global state that is shared across the applica
 
 import asyncio
 import concurrent.futures
-from typing import Any, Optional, TYPE_CHECKING
+from typing import Any, List, Optional, TYPE_CHECKING
+import warnings
 
 from pydantic import BaseModel, ConfigDict
 
 from mcp import ServerSession
+from mcp.server.fastmcp import FastMCP
 
 from opentelemetry import trace
 
@@ -34,6 +36,7 @@ from mcp_agent.tracing.token_counter import TokenCounter
 
 
 if TYPE_CHECKING:
+    from mcp_agent.agents.agent_spec import AgentSpec
     from mcp_agent.human_input.types import HumanInputCallback
     from mcp_agent.elicitation.types import ElicitationCallback
     from mcp_agent.executor.workflow_signal import SignalWaitCallback
@@ -41,6 +44,7 @@ if TYPE_CHECKING:
     from mcp_agent.app import MCPApp
 else:
     # Runtime placeholders for the types
+    AgentSpec = Any
     HumanInputCallback = Any
     ElicitationCallback = Any
     SignalWaitCallback = Any
@@ -66,6 +70,9 @@ class Context(BaseModel):
     session_id: str | None = None
     app: Optional["MCPApp"] = None
 
+    # Subagents
+    loaded_subagents: List["AgentSpec"] = []
+
     # Registries
     server_registry: Optional[ServerRegistry] = None
     task_registry: Optional[ActivityRegistry] = None
@@ -82,11 +89,19 @@ class Context(BaseModel):
     # Token counting and cost tracking
     token_counter: Optional[TokenCounter] = None
 
+    # Dynamic gateway configuration (per-run overrides via Temporal memo)
+    gateway_url: str | None = None
+    gateway_token: str | None = None
+
     model_config = ConfigDict(
         extra="allow",
         arbitrary_types_allowed=True,  # Tell Pydantic to defer type evaluation
     )
 
+    @property
+    def mcp(self) -> FastMCP | None:
+        return self.app.mcp if self.app else None
+
 
 async def configure_otel(
     config: "Settings", session_id: str | None = None
@@ -196,8 +211,8 @@ async def initialize_context(
 
     context.session_id = str(context.executor.uuid())
 
-    # Initialize token counter
-    context.token_counter = TokenCounter()
+    # Initialize token counter with engine hint for fast path checks
+    context.token_counter = TokenCounter(execution_engine=config.execution_engine)
 
     # Configure logging and telemetry
     context.tracing_config = await configure_otel(config, context.session_id)
@@ -275,6 +290,13 @@ def get_current_context() -> Context:
                 _global_context = loop.run_until_complete(initialize_context())
         except RuntimeError:
             _global_context = asyncio.run(initialize_context())
+
+        # Advisory: using a global context can cause cross-thread coupling
+        warnings.warn(
+            "get_current_context() created a global Context. "
+            "In multithreaded runs, instantiate an MCPApp per thread and use app.context instead.",
+            stacklevel=2,
+        )
     return _global_context
 
 
