@@ -98,15 +98,31 @@ class TemporalExecutor(Executor):
 
         @activity.defn(name=activity_name)
         async def wrapped_activity(*args, **local_kwargs):
+            """
+            Temporal activity wrapper that supports both payload styles:
+            - Single dict payload: wrapped_activity({"k": v, ...}) -> func(**payload)
+            - Varargs/kwargs payload: wrapped_activity(a, b, c, x=1) -> func(a, b, c, x=1)
+            """
             try:
-                if asyncio.iscoroutinefunction(func):
-                    return await func(**args[0])
-                elif asyncio.iscoroutine(func):
-                    return await func
+                # Prefer the legacy single-dict payload convention when applicable
+                if len(args) == 1 and isinstance(args[0], dict) and not local_kwargs:
+                    payload = args[0]
+                    if asyncio.iscoroutinefunction(func):
+                        return await func(**payload)
+                    elif asyncio.iscoroutine(func):
+                        return await func
+                    else:
+                        return func(**payload)
                 else:
-                    return func(**args[0])
+                    # Fall back to passing through varargs/kwargs directly
+                    if asyncio.iscoroutinefunction(func):
+                        return await func(*args, **local_kwargs)
+                    elif asyncio.iscoroutine(func):
+                        return await func
+                    else:
+                        return func(*args, **local_kwargs)
             except Exception as e:
-                # Handle exceptions gracefully
+                # Properly surface activity exceptions
                 raise e
 
         return wrapped_activity
@@ -163,6 +179,7 @@ class TemporalExecutor(Executor):
         activity_registry = self.context.task_registry
         activity_task = activity_registry.get_activity(activity_name)
 
+        # Config timeout takes priority over metadata timeout (per tests).
         schedule_to_close = self.config.timeout_seconds or execution_metadata.get(
             "schedule_to_close_timeout"
         )
@@ -170,15 +187,17 @@ class TemporalExecutor(Executor):
         if schedule_to_close is not None and not isinstance(
             schedule_to_close, timedelta
         ):
-            # Convert to timedelta if it's not already
+            # Convert numeric seconds to timedelta if needed
             schedule_to_close = timedelta(seconds=schedule_to_close)
 
         retry_policy = execution_metadata.get("retry_policy", None)
 
         try:
+            # Temporal's execute_activity accepts at most one positional arg;
+            # pass user args via the keyword-only 'args' to support multiple
             result = await workflow.execute_activity(
                 activity_task,
-                *args,
+                args=list(args) if args else None,
                 task_queue=self.config.task_queue,
                 schedule_to_close_timeout=schedule_to_close,
                 retry_policy=retry_policy,
