@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import Any, Callable, List, Literal, Sequence, Tuple
+from typing import Any, Callable, List, Literal, Sequence, Tuple, overload
 import os
 import re
 import json
@@ -65,6 +65,17 @@ def agent_from_spec(spec: AgentSpec, context: Context | None = None) -> Agent:
     )
 
 
+@overload
+def create_llm(
+    agent: Agent | AgentSpec,
+    provider: str | None = "openai",
+    model: str | ModelPreferences | None = None,
+    request_params: RequestParams | None = None,
+    context: Context | None = None,
+) -> AugmentedLLM: ...
+
+
+@overload
 def create_llm(
     agent_name: str,
     server_names: List[str] | None = None,
@@ -73,17 +84,47 @@ def create_llm(
     model: str | ModelPreferences | None = None,
     request_params: RequestParams | None = None,
     context: Context | None = None,
+) -> AugmentedLLM: ...
+
+
+def create_llm(
+    agent: Agent | AgentSpec | None = None,
+    agent_name: str | None = None,
+    server_names: List[str] | None = None,
+    instruction: str | None = None,
+    provider: str = "openai",
+    model: str | ModelPreferences | None = None,
+    request_params: RequestParams | None = None,
+    context: Context | None = None,
 ) -> AugmentedLLM:
-    agent = agent_from_spec(
-        AgentSpec(
-            name=agent_name, instruction=instruction, server_names=server_names or []
-        ),
+    """
+    Create an Augmented LLM from an agent, agent spec, or agent name.
+    """
+    if isinstance(agent_name, str):
+        # Handle the case where first argument is agent_name (string)
+        agent_obj = agent_from_spec(
+            AgentSpec(
+                name=agent_name,
+                instruction=instruction,
+                server_names=server_names or [],
+            ),
+            context=context,
+        )
+    elif isinstance(agent, AgentSpec):
+        # Handle AgentSpec case
+        agent_obj = agent_from_spec(agent, context=context)
+    else:
+        # Handle Agent case
+        agent_obj = agent
+
+    factory = _llm_factory(
+        provider=provider,
+        model=model,
+        request_params=request_params,
         context=context,
     )
-    factory = _llm_factory(
-        provider=provider, model=model, request_params=request_params, context=context
-    )
-    return factory(agent=agent)
+
+    return factory(agent=agent_obj)
 
 
 async def create_router_llm(
@@ -93,7 +134,7 @@ async def create_router_llm(
     functions: List[Callable] | None = None,
     routing_instruction: str | None = None,
     name: str | None = None,
-    provider: SupportedRoutingProviders = "openai",
+    provider: SupportedLLMProviders = "openai",
     model: str | ModelPreferences | None = None,
     request_params: RequestParams | None = None,
     context: Context | None = None,
@@ -155,8 +196,22 @@ async def create_router_llm(
             **kwargs,
         )
     else:
-        raise ValueError(
-            f"Unsupported routing provider: {provider}. Currently supported providers are: ['openai', 'anthropic']. To request support, please create an issue at https://github.com/lastmile-ai/mcp-agent/issues"
+        factory = _llm_factory(
+            provider=provider,
+            model=model,
+            request_params=request_params,
+            context=context,
+        )
+
+        return await LLMRouter.create(
+            name=name,
+            llm_factory=factory,
+            server_names=server_names,
+            agents=normalized_agents,
+            functions=functions,
+            routing_instruction=routing_instruction,
+            context=context,
+            **kwargs,
         )
 
 
@@ -944,9 +999,20 @@ def _llm_factory(
     request_params: RequestParams | None = None,
     context: Context | None = None,
 ) -> Callable[[Agent], AugmentedLLM]:
+    # Allow model to come from an explicit string, request_params.model,
+    # or request_params.modelPreferences (to run selection) in that order.
+    # Compute the chosen model by precedence:
+    # 1) explicit model_name from _select_provider_and_model (includes ModelPreferences)
+    # 2) provider default from provider_cls.get_provider_config(context)
+    # 3) provider hardcoded fallback
+    model_selector_input = (
+        model
+        or getattr(request_params, "model", None)
+        or getattr(request_params, "modelPreferences", None)
+    )
     prov, model_name = _select_provider_and_model(
         provider=provider,
-        model=model or getattr(request_params, "model", None),
+        model=model_selector_input,
         context=context,
     )
     provider_cls = _get_provider_class(prov)
@@ -960,9 +1026,28 @@ def _llm_factory(
             return RequestParams(modelPreferences=model)
         return None
 
+    # Merge provider-selected or configured default model into RequestParams if missing.
+    effective_params: RequestParams | None = request_params
+    if effective_params is not None:
+        chosen_model: str | None = model_name
+
+        if not chosen_model:
+            cfg_obj = None
+            try:
+                cfg_obj = provider_cls.get_provider_config(context)
+            except Exception:
+                cfg_obj = None
+            if cfg_obj is not None:
+                chosen_model = getattr(cfg_obj, "default_model", None)
+
+        # If the user did not specify a model in RequestParams, but provided other
+        # overrides (maxTokens, temperature, etc.), fill in the model only.
+        if getattr(effective_params, "model", None) is None and chosen_model:
+            effective_params.model = chosen_model
+
     return lambda agent: provider_cls(
         agent=agent,
-        default_request_params=request_params or _default_params(),
+        default_request_params=effective_params or _default_params(),
         context=context,
     )
 
