@@ -149,6 +149,10 @@ class AnthropicAugmentedLLM(AugmentedLLM[MessageParam, Message]):
             use_history=True,
         )
 
+    @classmethod
+    def get_provider_config(cls, context):
+        return getattr(getattr(context, "config", None), "anthropic", None)
+
     @track_tokens()
     async def generate(
         self,
@@ -180,7 +184,9 @@ class AnthropicAugmentedLLM(AugmentedLLM[MessageParam, Message]):
                 AnthropicConverter.convert_mixed_messages_to_anthropic(message)
             )
 
-            list_tools_result = await self.agent.list_tools()
+            list_tools_result = await self.agent.list_tools(
+                tool_filter=params.tool_filter
+            )
             available_tools: List[ToolParam] = [
                 {
                     "name": tool.name,
