@@ -128,6 +128,10 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
             use_history=True,
         )
 
+    @classmethod
+    def get_provider_config(cls, context):
+        return getattr(getattr(context, "config", None), "azure", None)
+
     @track_tokens()
     async def generate(self, message, request_params: RequestParams | None = None):
         """
@@ -160,7 +164,7 @@ class AzureAugmentedLLM(AugmentedLLM[MessageParam, ResponseMessage]):
 
             messages.extend(AzureConverter.convert_mixed_messages_to_azure(message))
 
-            response = await self.agent.list_tools()
+            response = await self.agent.list_tools(tool_filter=params.tool_filter)
 
             tools: list[ChatCompletionsToolDefinition] = [
                 ChatCompletionsToolDefinition(
