@@ -134,6 +134,10 @@ class OpenAIAugmentedLLM(
             use_history=True,
         )
 
+    @classmethod
+    def get_provider_config(cls, context):
+        return getattr(getattr(context, "config", None), "openai", None)
+
     @classmethod
     def convert_message_to_message_param(
         cls, message: ChatCompletionMessage, **kwargs
@@ -189,7 +193,9 @@ class OpenAIAugmentedLLM(
                 )
             messages.extend((OpenAIConverter.convert_mixed_messages_to_openai(message)))
 
-            response: ListToolsResult = await self.agent.list_tools()
+            response: ListToolsResult = await self.agent.list_tools(
+                tool_filter=params.tool_filter
+            )
             available_tools: List[ChatCompletionToolParam] = [
                 ChatCompletionToolParam(
                     type="function",
